/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Configure method for PINT
 * third release, version 0.3
 */

#include <sys/types.h>
#include <sys/cfgdb.h>
#include <sys/cfgodm.h>
#include <sys/sysconfig.h>
#include <sys/device.h>
#include <sys/mode.h>
#include <sys/errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <cf.h>
#include <sys/stat.h>
#include <sys/scsi.h>
#include <sys/watchdog.h>
#ifdef IOSTAT			/* define IOSTAT to use "iostat -d scan0" */
#include <sys/iostat.h>
#endif
#include "scandd.h"

#define USAGE(S)	(fprintf(stderr, "Usage: %s -l name [ -1 | -2 ]\n", (S)), exit(E_ARGS))


#define UNDO0		0
#define UNDO1		1
#define UNDO2		2
#define UNDO3		3

#define CDEV_MODE	(S_IFCHR|0444)

int		lock_id;
int		phase = RUNTIME_CFG;
struct cfg_dd	cfg;
struct PdDv	pddv;
struct CuDv	cudv;
char	       *logical_name = NULL;

/*
 * function prototypes
 */
void close_odm_exit(int err, int undo);
int scsi_inq_test(char *parent, char *connwhere, char *model);


main(int argc, char *argv[])
{
  char	 search_string[0x100];
  char	 *odm_path;
  char   odm_lock_path[1024];
  char	 devpath[0x100];
  long	 majorno, minor;

  char parent_dev[1024];	/* pathname to the parent's special file */
  struct stat parent_stat;	/* parent's special file inode data */
  SCAN_DDS sdds;		/* uiomove() puts the DDS in here */

  struct CuAt	*cuat;		/* ODM handle */
  struct PdAt   pdat;		/* ODM returns model name here */
  struct CuDv   parent_obj;	/* ODM returns parent adapter here */
  struct CuDv   connection;	/* ODM returns on connection here */

  extern int   optind;
  extern char *optarg;
  int    c;			/* for getopt() return */

  int    n;			/* getattr() count return */

  while ((c = getopt(argc, argv, "l:12")) != EOF) {
    switch (c) {
    case 'l':
      if (logical_name != NULL)
	USAGE(*argv);
      logical_name = optarg;
      break;
    case '1':
      if (phase != RUNTIME_CFG)
	USAGE(*argv);
      phase = PHASE1;
      break;
    case '2':
      if (phase != RUNTIME_CFG)
	USAGE(*argv);
      phase = PHASE2;
      break;
    default:
      USAGE(*argv);
    }
  }

  if (logical_name == NULL)	/* logical name must be specified */
    exit(E_LNAME);


  /* initialize and lock ODM */

  if (odm_initialize() < 0)
    exit(E_ODMINIT);

  if ((odm_path = getenv("ODMDIR")) == NULL) {
    fprintf(stderr, "%s: ODMDIR environment variable is undefined.\n",
	    argv[0]);
    exit(E_ODMLOCK);
  }
  strcpy(odm_lock_path, odm_path);
  strcat(odm_lock_path, "/config_lock");
  if ((lock_id = odm_lock(odm_lock_path, 10)) == -1)
    close_odm_exit(E_ODMLOCK, UNDO0);

  /* get customized device entry from CuDv */

  sprintf(search_string, "name = '%s'", logical_name);
  switch ((int)odm_get_first(CuDv_CLASS, search_string, &cudv)) {
  case 0:
    close_odm_exit(E_NOCuDv, UNDO0);
  case -1:
    close_odm_exit(E_ODMGET, UNDO0);
  }

  /* get predefined device from PdDv */

  sprintf(search_string, "uniquetype = '%s'", cudv.PdDvLn_Lvalue);
  switch ((int)odm_get_first(PdDv_CLASS, search_string, &pddv)) {
  case 0:
    close_odm_exit(E_NOPdDv, UNDO0);
  case -1:
    close_odm_exit(E_ODMGET, UNDO0);
  }

  /* if the system is booting work the LED display */

  if (phase != RUNTIME_CFG)
    setleds(pddv.led);

  /* only configure "DEFINED" devices */

  if (cudv.status == DEFINED) {

    /*  get the tunable parameters from the ODM  */

    if ((cuat = getattr(logical_name, "window_width", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.window_width = atol(cuat->value);

    if ((cuat = getattr(logical_name, "window_length", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.window_length = atol(cuat->value);
 
   if ((cuat = getattr(logical_name, "x_resolution", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.x_resolution = (ushort)atoi(cuat->value);

    if ((cuat = getattr(logical_name, "y_resolution", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.y_resolution = (ushort)atoi(cuat->value);
    
    if ((cuat = getattr(logical_name, "x_origin", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.x_origin = atol(cuat->value);

    if ((cuat = getattr(logical_name, "y_origin", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    sdds.y_origin = atol(cuat->value);

    if ((cuat = getattr(logical_name, "image_mode", FALSE, &n)) == NULL)
      close_odm_exit(E_NOATTR, UNDO0);
    if (strcmp(cuat->value, "grayscale") == 0)
      sdds.image_mode = GRAYSCALE;
    else if (strcmp(cuat->value, "dithered") == 0)
      sdds.image_mode = DITHERED_MONOCHROME;
    else if (strcmp(cuat->value, "monochrome") == 0)
      sdds.image_mode = BINARY_MONOCHROME;
    else if (strcmp(cuat->value, "red") == 0)
      sdds.image_mode = RED;
    else if (strcmp(cuat->value, "green") == 0)
      sdds.image_mode = GREEN;
    else if (strcmp(cuat->value, "blue") == 0)
      sdds.image_mode = BLUE;
    else if (strcmp(cuat->value, "color") == 0)
      sdds.image_mode = COLOR;

    if ((cuat = getattr(logical_name, "brightness", FALSE, &n)) != NULL)
      sdds.brightness = (uchar)atoi(cuat->value);

    if ((cuat = getattr(logical_name, "contrast", FALSE, &n)) != NULL)
      sdds.contrast = (uchar)atoi(cuat->value);

    if ((cuat = getattr(logical_name, "use_adf", FALSE, &n)) != NULL)
      sdds.use_adf = (uchar)(strcmp(cuat->value, "no"));

    /* populate DDS with 'connwhere' */

    sdds.scsi_id = (uchar)(cudv.connwhere[0] - '0');
    sdds.lun     = (uchar)(cudv.connwhere[1] - '0');

    /* populate DDS with logical name (scan0) */

    strncpy(sdds.logical_name, logical_name, 8);

    /* populate DDS with scanner type code */

    if (strcmp(pddv.type, "ricoh_is410") == 0)
      sdds.scanner_type = RICOH_IS410;
    else if (strcmp(pddv.type, "fujitsu") == 0)
      sdds.scanner_type = FUJITSU;
    else if (strcmp(pddv.type, "ricoh_fs1") == 0)
      sdds.scanner_type = RICOH_FS1;
    else if (strcmp(pddv.type, "sharp_jx600") == 0)
      sdds.scanner_type = SHARP_JX600;
    else if (strcmp(pddv.type, "ricoh_is50") == 0)
      sdds.scanner_type = RICOH_IS50;
    else if (strcmp(pddv.type, "ibm_2456") == 0)
      sdds.scanner_type = IBM_2456;
    else if (strcmp(pddv.type, "umax_uc630") == 0)
      sdds.scanner_type = UMAX_UC630;
    else if (strcmp(pddv.type, "umax_ug630") == 0)
      sdds.scanner_type = UMAX_UG630;
    else if (strcmp(pddv.type, "hp_scanjet_IIc") == 0)
      sdds.scanner_type = HP_SCANJET_IIC;

    /* check the parent */

    sprintf(search_string, "name = '%s'", cudv.parent);
    switch ((int)odm_get_first(CuDv_CLASS, search_string, &parent_obj)) {
    case 0:
      close_odm_exit(E_NOCuDvPARENT, UNDO0);
    case -1:
      close_odm_exit(E_ODMGET, UNDO0);
    }

    /* test for parent's availability */
    if (parent_obj.status != AVAILABLE)
      close_odm_exit(E_PARENTSTATE, UNDO0); /* can't continue w.o./parent */

    /* check the connection */

    /*  use SCSI INQUIRY command to see if the scanner matches the ODM  */

    /* get the model name from the PdAt */
    sprintf(search_string, "uniquetype = '%s' and attribute = 'model_name'",
	    pddv.uniquetype);
    switch ((int)odm_get_first(PdAt_CLASS, search_string, &pdat)) {
    case 0:
      close_odm_exit(E_NOPdOBJ, UNDO0);
    case -1:
      close_odm_exit(E_ODMGET, UNDO0);
    }

    while (TRUE) {
      if (scsi_inq_test(cudv.parent, cudv.connwhere, pdat.deflt))
	break;

      /* check all model names in the ODM */

      switch ((int)odm_get_next(PdAt_CLASS, &pdat)) {
      case 0:
	close_odm_exit(E_NODETECT, UNDO0);
      case -1:
	close_odm_exit(E_ODMGET, UNDO0);
      }
    }

    /* test connection, reject if already in use */

    sprintf(search_string,
	    "parent = '%s' AND connwhere = '%s' AND status = %d",
	    cudv.parent, cudv.connwhere, AVAILABLE);
    switch ((int)odm_get_first(CuDv_CLASS, search_string, &connection)) {
    case -1:
      close_odm_exit(E_ODMGET, UNDO0);
    case 0:
      break;		/* if nothing is connected then we are fine */
    default:
      close_odm_exit(E_AVAILCONNECT, UNDO0);
    }

    /* if the driver exists then load it */

    if (*pddv.DvDr != 0) {
      if ((cfg.kmid = loadext(pddv.DvDr, TRUE, FALSE)) == NULL)
	close_odm_exit(E_LOADEXT, UNDO0);

      /*  get the major & minor numbers  */

      if ((majorno = genmajor(pddv.DvDr)) == -1)
	close_odm_exit(E_MAJORNO, UNDO1);

      minor = atol(&logical_name[4]); /* minor is logical name's number */

      /*   make the special file   */

      cfg.devno  = makedev(majorno, minor);
      sprintf(devpath, "/dev/%s", logical_name);

      if (mknod(devpath, CDEV_MODE, cfg.devno) < 0 && errno == EEXIST) {
	if (unlink(devpath) < 0 || mknod(devpath, CDEV_MODE, cfg.devno) < 0)
	  close_odm_exit(E_MKSPECIAL, UNDO2);
      }

      /* complete the DDS */

      /* put the parent adapter special file devno into the DDS */
      strcpy(parent_dev, "/dev/");
      strcat(parent_dev, cudv.parent);
      if (stat(parent_dev, &parent_stat) < 0)
	close_odm_exit(E_DDS, UNDO2);
      sdds.parent_devno = parent_stat.st_rdev;

      cfg.ddsptr = (char *)&sdds;
      cfg.ddslen = sizeof(sdds);

      /*  call the driver's initialization routine  */

      cfg.cmd = CFG_INIT;
      if (sysconfig(SYS_CFGDD, &cfg, sizeof(struct cfg_dd)) == -1)
	close_odm_exit(E_CFGINIT, UNDO2);

    }  /* test for presence of device driver */

    /* mark device as available in CuDv */

    cudv.status = AVAILABLE;
    if (odm_change_obj(CuDv_CLASS, &cudv) < 0)
      close_odm_exit(E_ODMUPDATE, UNDO3);

  } /* test if device is AVAILABLE */

  close_odm_exit(E_OK, UNDO0);
}

/*
 * scsi_inq_test: look for scanner using SCSI INQUIRY command
 * takes the parent device name "/dev/scsi0", a connection code "connwhere"
 *  and the model name we are looking for
 * returns non-zero if it finds that the model string matches
 *  or a zero if it does not match, or if any errors occur.
 */

int
scsi_inq_test(char *parent, char *connwhere, char *model)
{
  int scsi_fd;
  struct sc_inquiry sc_inq;
  uchar inq_response[128];	/* response from fp_ioctl SCIOINQU */
  char parent_dev[128];
  int back_out = 0;		/* for undoing stuff when errors happen */
  int rc = 0;			/* return from this funtion */

  strcpy(parent_dev, "/dev/");
  strcat(parent_dev, parent);

  /* open the host adapter */
  if ((scsi_fd = open(parent_dev, O_RDONLY)) < 0) {

#ifdef DEBUG
    fprintf(stderr, "open of %s failed -- errno = %d", parent_dev, errno);
#endif

    return (1);
  } else {
    ++back_out;			/* now a close(scsi_fd) is necessary */
  }

  /* send an ioctl START to the host adapter */

  if (ioctl(scsi_fd, SCIOSTART,
	    IDLUN((uchar)(connwhere[0]-'0'), (uchar)(connwhere[1]-'0'))) < 0) {

#ifdef DEBUG
    fprintf(stderr, "ioctl SCIOSTART on  %s failed -- errno = %d",
	    parent_dev, errno);
#endif

    goto inq_out;
  } else {
    ++back_out;			/* now a SCIOSTOP is necessary */
  }

  /* send an ioctl INQUIRY to the host adapter */
  /* fill out the sc_inquiry struct */
  sc_inq.scsi_id	= (uchar)(connwhere[0] - '0');
  sc_inq.lun_id		= (uchar)(connwhere[1] - '0');
  sc_inq.rsv1		= (uchar)0;
  sc_inq.inquiry_len	= (uchar)(sizeof(inq_response) - 1);
  sc_inq.inquiry_ptr	= inq_response;
  sc_inq.rsv2		= (uchar)0;
  sc_inq.rsv3		= (uchar)0;
  sc_inq.rsv4		= (uchar)0;
  if (strncmp(model, "FS1 S",  5) == 0 ||
      strncmp(model, "IS50",   4) == 0 ||
      strncmp(model, "UC630",  5) == 0 ||
      strncmp(model, "UG630",  5) == 0 ||
      strncmp(model, "C1750A", 6) == 0 ||
      strncmp(model, "C2500A", 6) == 0)
    sc_inq.flags	= (uchar)SC_ASYNC;
  else
    sc_inq.flags	= (uchar)0;
  sc_inq.rsv5		= (uint)0;

  if (ioctl(scsi_fd, SCIOINQU, &sc_inq) < 0) {
#ifdef DEBUG
    fprintf(stderr, "ioctl SCIOINQU on %s failed -- errno = %d",
	    connwhere, errno);
#endif
    goto inq_out;
  } else {
#ifdef DEBUG
    fprintf(stderr, "model: '%s'  response: '%s'\n", model, &inq_response[16]);
#endif
    if (strncmp(model, &inq_response[16], strlen(model)) == 0)
      rc = 1;			/* ta-da! */
  }

  /****  exit scsi_inq_test performing any cleanup  ***/

 inq_out:
  switch (back_out) {
  case 2:			/* a SCIOSTOP is necessary */

    (void)ioctl(scsi_fd, SCIOSTOP,   /* hope it worked! */
	  IDLUN((uchar)(connwhere[0]-'0'), (uchar)(connwhere[1]-'0')));

				/* fall through to 1 */

  case 1:			/* a close(scsi_fd) is necessary */
    close(scsi_fd);		/* fall through to 0 */

  case 0:			/* no cleanup necessary */
    break;

  default:
    fprintf(stderr, "cfgscan in scsi_inq_test(): bad back_out value: %d\n",
	    back_out);
  }
  return (rc);
}

/* clean up and exit */

void
close_odm_exit(int err, int undo)
{
  odm_unlock(lock_id);
  odm_terminate();

  switch (undo) {
  case UNDO0:
    break;

  case UNDO3:			  /* Terminate Device */
    cfg.kmid = (mid_t) 0;
    cfg.ddsptr = NULL;
    cfg.ddslen = (int) 0;
    cfg.cmd = CFG_TERM;
    (void) sysconfig(SYS_CFGDD, &cfg, sizeof(struct cfg_dd));
				  /* fall through to UNDO2... */

  case UNDO2:			  /* release major number & unload driver */
    reldevno(logical_name, TRUE); /* fall through to UNDO1... */

  case UNDO1:			  /* Unload driver */
    (void) loadext(pddv.DvDr, FALSE, FALSE);
    break;
  }
  exit(err);
}
