/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* 
 * mkinventory.c: given an apply list "al" create an inventory file "inventory"
 */

#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>

void gen_inventory(FILE *, FILE *, char *);
char *get_option_name(FILE *);

main()
{
  FILE *al, *inventory, *lpp_name, *opt_inv;
  char line[128];
  char *option_name;

  if ((lpp_name = fopen("../../../lpp_name", "r")) == NULL) {
    perror("mkinventory: can't open 'lpp_name'");
    exit(1);
  }

  option_name = get_option_name(lpp_name);

  fclose(lpp_name);

  sprintf(line, "%s.inv.u", option_name);

  if ((opt_inv = fopen(line, "w")) == NULL) {
    perror("mkinventory: can't open 'option.inv.u'");
    exit(1);
  }

  if ((al = fopen("al", "r")) == NULL) {
    perror("mkinventory: can't open 'al'");
    exit(1);
  }

  if ((inventory = fopen("inventory", "w")) == NULL) {
    perror("mkinventory: can't open 'inventory'");
    exit(1);
  }

  chdir("../../..");

  while (fgets(line, 128, al) != NULL) {
    line[strlen(line) - 1] = '\0'; /* eat newline */
    gen_inventory(inventory, opt_inv, line);
  }

  fclose(al);
  fclose(inventory);
  fclose(opt_inv);

  exit(0);
}

void gen_inventory(FILE *inv, FILE *opt_inv, char *item)
{
  struct stat stat_buf;
  struct passwd *pwp;
  struct group *grp;
  FILE *sum;
  char sum_value[128];
  char abs_item[128];
  char err[128];
  char cmd[128];
  char *p;

#if 1
  /* XXX -> stupid hack need because the lppchk program is run after
   *        these files get renamed and therefore won't find them
   */
  if (strcmp(item, "./usr/bin/X11/ice_scan.3.1") == 0 ||
      strcmp(item, "./usr/bin/X11/ice_scan.3.2") == 0)
    return;
#endif

  if (stat(item, &stat_buf) < 0) {
    sprintf(err, "mkinventory: can't stat %s", item);
    perror(err);
    exit(2);
  }

  if ((pwp = getpwuid(stat_buf.st_uid)) == NULL) {
    fprintf(stderr, "mkinventory: can't get password entry for uid %d\n",
	    stat_buf.st_uid);
    exit(2);
  }

  if ((grp = getgrgid(stat_buf.st_gid)) == NULL) {
    fprintf(stderr, "mkinventory: can't get group entry for gid %d\n",
	    stat_buf.st_gid);
    exit(2);
  }

  sprintf(cmd, "sum %s", item);
  if ((sum = popen(cmd, "r")) == NULL) {
    sprintf(err, "mkinventory: can't run %s", cmd);
    perror(err);
    exit(2);
  }
  fgets(sum_value, 128, sum);
  pclose(sum);
  p = strrchr(sum_value, ' ');	/* get rid of the program name */
  ++p;
  *p = '\0';

  if (item[0] == '.') {
    strcpy(abs_item, "/");
    strcat(abs_item, &item[2]);
  } else
    strcpy(abs_item, item);

  fprintf(inv, "%s:\n", abs_item);
  fprintf(inv, "\towner = %s\n", pwp->pw_name);
  fprintf(inv, "\tgroup = %s\n", grp->gr_name);
  fprintf(inv, "\ttype = FILE\n");
  fprintf(inv, "\tmode = ");
  if (stat_buf.st_mode & S_ISUID)
    fprintf(inv, "SUID,");
  if (stat_buf.st_mode & S_ISGID)
    fprintf(inv, "SGID,");
  fprintf(inv, "%o\n", stat_buf.st_mode & ~(_S_IFMT|S_ISUID|S_ISGID));
  fprintf(inv, "\tsize = %d\n", stat_buf.st_size);
  fprintf(inv, "\tchecksum = \"%s\"\n", sum_value);
  fprintf(inv, "\n");

  fprintf(opt_inv, "%s:\n", abs_item);
  fprintf(opt_inv, "\towner = %s\n", pwp->pw_name);
  fprintf(opt_inv, "\tgroup = %s\n", grp->gr_name);
  fprintf(opt_inv, "\ttype = FILE\n");
  fprintf(opt_inv, "\tmode = ");
  if (stat_buf.st_mode & S_ISUID)
    fprintf(opt_inv, "SUID,");
  if (stat_buf.st_mode & S_ISGID)
    fprintf(opt_inv, "SGID,");
  fprintf(opt_inv, "%o\n", stat_buf.st_mode & ~(_S_IFMT|S_ISUID|S_ISGID));
  fprintf(opt_inv, "\toldpath = %s\n", abs_item);
  fprintf(opt_inv, "\tsize = %d\n", stat_buf.st_size);
  fprintf(opt_inv, "\tchecksum = \"%s\"\n", sum_value);
  fprintf(opt_inv, "\n");
}

char *get_option_name(FILE *lpp_name)
{
  static char option_name[128];
  char *p;

  fgets(option_name, 128, lpp_name);
  fgets(option_name, 128, lpp_name);
  p = strchr(option_name, ' ');
  *p = '\0';

  return (option_name);
}
