/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * defines for different types of scanners
 * third release, version 0.3
 */

#define RICOH_IS410	1
#define FUJITSU		2	/* obsolescent #define */
#define FUJITSU_M3096G  2	/* prefered #define */
#define HP_SCANJET_IIC  3
#define RICOH_FS1	4
#define SHARP_JX600	5
#define RICOH_IS50	6
#define IBM_2456	7
#define UMAX_UC630	8
#define UMAX_UG630      9


/*
 * defines for paper sizes
 */
#define STANDARD_PAPER_SIZE	0x80
#define NONSTANDARD_PAPER_SIZE	0xc0
#define PORTRAIT		0x00
#define LANDSCAPE		0x10
#define LETTER			0x07
#define DOUBLE_LETTER		0x06
#define LEGAL			0x0f

/*
 * defines for IMAGE_COMPOSITION field
 */
#define BINARY_MONOCHROME   0
#define DITHERED_MONOCHROME 1
#define GRAYSCALE 2
#define COLOR	  5
#define RED	  103
#define GREEN	  104
#define BLUE	  105


/*
 * defines for HALFTONE_PATTERN field
 */
#define BUILTIN_HALFTONE_PATTERN	0
#define DOWNLOADABLE_HALFTONE_PATTERN	1

/*
 * scsi opcodes not provided in <sys/scsi.h>
 */
#define SCSI_SET_WINDOW		0x24
#define SCSI_GET_WINDOW		0x25
#define SCSI_TRIGGER_SCAN       0x1b
#define SCSI_GET_BUFFER_STATUS  0x34
#define SCSI_OBJECT_POSITION    0x31
#define SCSI_GET_IMAGE_STATUS   0x0f
/* read request with special data type tells sharp to abort read mode */
#define SCSI_SHARP_READ_ABORT   SCSI_READ_EXTENDED

/*
 *  defines for sc_buf.scsi_command.scsi_cmd.scsi_bytes
 *  SCSI command block
 */
#define SCB2 0
#define SCB3 1
#define SCB4 2
#define SCB5 3
#define SCB6 4
#define SCB7 5
#define SCB8 6
#define SCB9 7

/* sizes for mode parameter's base_measurement_unit */

#define INCHES 0
#define MILLIMETERS 1
#define POINTS 2

/*
 *  misc defines
 */
#define SCANNER_HASHSIZE 0x0f	/* for scanner_list table */
#define SCAN_STOP_UNIT	0
#define SCAN_START_UNIT 1
#define SCAN_PASS_THRU		  0xEF	  /* cmd state for pass-thru ioctl */

#define ADAPTER_FATAL		  0xFD /* adapter_error for you're hosed */
#define ADAPTER_RETRY		  0xFE /* adapter_error for retry cmd  */
#define ADAPTER_RESET		  0xFF /* adapter_error for reset dev  */

#define MAX_TUR_RETRIES		  3   /* number of Test Unit Ready retries */

/* misc bit flags */
#define PF_BIT 0x10		/* for mode select */
#define WAIT_BIT (uchar)(0x01)	/* for get data buffer status */
#define LOAD_BIT (uchar)(0x01)	/* for object position */

#define SCANNING_MEASUREMENT_PARAMETERS (uchar)(3)

/*
 * the (ever-increasing) Device Dependent Structure (DDS)
 */

typedef struct scan_dds SCAN_DDS;

struct scan_dds {
  uchar  scsi_id;		/* SCSI id from 0 to 7 */
  uchar  lun;			/* Logical Unit Number */
  dev_t  parent_devno;		/* major-minor combo from parent adapter */
  char   logical_name[8];	/* "scan0" for instance */
  uint	 scanner_type;		/* see defines above */
  ulong  window_width;		/* width in 1/2000ths of an inch */
  ulong  window_length;		/* length in 1/2000ths of an inch */
  ushort x_resolution;		/* horizontal resolution in dots-per-inch */
  ushort y_resolution;		/* vertical resolution in dots-per-inch */
  ulong  x_origin;		/* horizontal coordinate of upper left corner */
  ulong  y_origin;		/* vertical coordinate of upper left corner */
  uchar  image_mode;		/* type of image data sent by scanner */
  uchar  brightness;		/* image brightness control */
  uchar  contrast;		/* image contrast control */
  uchar  use_adf;		/* use Automatic Document Feeder */
};

/*
 *  SCSI_MODE_SELECT parameter list
 */
struct ricoh_mode_params {
  /* header */
  uchar mode_data_length;
  uchar medium_type;
  uchar device_specific_parameter;
  uchar block_descriptor_length;

  /* page descriptor */
  uchar page_code;
  uchar paramter_length;
  uchar base_measurement_unit;
  uchar reserved1;
  uchar measurement_unit_devisor_msb;
  uchar measurement_unit_devisor_lsb;
  uchar reserved2;
  uchar reserved3;

  /* ADF support */
  uchar adf_page_code;
  uchar adf_parameter_length;
  uchar adf_control;
  uchar reserved4;
  uchar reserved5;
  uchar reserved6;
  uchar reserved7;
  uchar reserved8;
};


/*
 * arrays & defines for setting scanner windows
 */

/* common header... */

#define window_desc_header_size 8

#define WDH_RESERVED0				0
#define WDH_RESERVED1				1
#define WDH_RESERVED2				2
#define WDH_RESERVED3				3
#define WDH_RESERVED4				4
#define WDH_RESERVED5				5
#define WDH_WINDOW_DESCRIPTOR_BLOCK_LENGTH_MSB	6
#define WDH_WINDOW_DESCRIPTOR_BLOCK_LENGTH_LSB	7

/* common portions of the window descriptor... */

#define window_desc_size 64

#define WD_WINDOW_ID		8
#define WD_RESERVED00		9
#define WD_X_RES_MSB		10
#define WD_X_RES_LSB		11
#define WD_Y_RES_MSB		12
#define WD_Y_RES_LSB		13
#define WD_UPPER_LEFT_X_MSB	14		/* Most Significant Byte */
#define WD_UPPER_LEFT_X_MHB	15		/* Middle High Byte */
#define WD_UPPER_LEFT_X_MLB	16		/* Middle Low Byte */
#define WD_UPPER_LEFT_X_LSB	17		/* Least Significant Byte */
#define WD_UPPER_LEFT_Y_MSB	18
#define WD_UPPER_LEFT_Y_MHB	19
#define WD_UPPER_LEFT_Y_MLB	20
#define WD_UPPER_LEFT_Y_LSB	21
#define WD_WIDTH_MSB		22
#define WD_WIDTH_MHB		23
#define WD_WIDTH_MLB		24
#define WD_WIDTH_LSB		25
#define WD_LENGTH_MSB		26
#define WD_LENGTH_MHB		27
#define WD_LENGTH_MLB		28
#define WD_LENGTH_LSB		29
#define WD_BRIGHTNESS		30
#define WD_THRESHOLD		31
#define WD_CONTRAST		32
#define WD_IMAGE_COMPOSTION	33
#define WD_BITS_PER_PIXEL	34

/* damn incompatibility with HALFTONEs... */

#define WDF_HALFTONE_PATTERN_MSB 35 /* Fujitsu style */
#define WDF_HALFTONE_PATTERN_LSB 36

#define WDR_HALFTONE_CODE       35 /* Ricoh style */
#define WDR_HALFTONE_ID         36

/* back to common stuff... */

#define WD_RIF_AND_PADDING	37
#define RIF		0x80	/* bitmask for RIF bit */
#define PADDING_TYPE	0x03	/* bitmask for padding type bits */

#define WD_BIT_ORDERING_MSB	38
#define WD_BIT_ORDERING_LSB	39
#define WD_COMPRESSION_TYPE	40
#define WD_COMPRESSION_ARG	41
#define WD_RESERVED01		42
#define WD_RESERVED02		43
#define WD_RESERVED03		44
#define WD_RESERVED04		45
#define WD_RESERVED05		46
#define WD_RESERVED06		47

/* Fujitsu specific window descriptor stuff... */

#define WDF_IDENTIFICATION_CODE 48
#define WDF_RESERVED07		49
#define WDF_OUTLINE		50
#define WDF_EMPHASIS		51
#define WDF_MIXED		52
#define WDF_MIRRORING		53
#define WDF_RESERVED08		54
#define WDF_RESERVED09		55
#define WDF_RESERVED10		56
#define WDF_RESERVED11		57
#define WDF_RESERVED12		58
#define WDF_SUBWINDOW_LIST_MSB	59
#define WDF_SUBWINDOW_LIST_LSB	60
#define WDF_PAPER_SIZE		61
#define WDF_X_MSB		62
#define WDF_X_MHB		63
#define WDF_X_MLB		64
#define WDF_X_LSB		65
#define WDF_Y_MSB		66
#define WDF_Y_MHB		67
#define WDF_Y_MLB		68
#define WDF_Y_LSB		69
#define WDF_RESERVED13		70
#define WDF_RESERVED14		71

/* Ricoh specific window descriptor stuff... */

#define WDR_RESERVED01          48
#define WDR_RESERVED02          49
#define WDR_MRIF_FILTERING_GAMMA_ID 50
#define WDR_RESERVED03          51
#define WDR_RESERVED04          52
#define WDR_BINARY_FILTER       53
#define WDR_RESERVED05          54
#define WDR_RESERVED06          55
#define WDR_RESERVED07          56
#define WDR_RESERVED08          58
#define WDR_RESERVED09          58
#define WDR_RESERVED10          59
#define WDR_RESERVED11          60
#define WDR_RESERVED12          61
#define WDR_RESERVED13          62
#define WDR_RESERVED14          63
#define WDR_RESERVED15          64
#define WDR_RESERVED16          65
#define WDR_RESERVED17          66
#define WDR_RESERVED18          67
#define WDR_RESERVED19          68
#define WDR_RESERVED20          69
#define WDR_RESERVED21          70
#define WDR_RESERVED22          71

/* UMAX specific window descriptor stuff... */
#define WDU_SPEED		48
#define WDU_SELECT_COLOR	49
#define WDU_HIGHLIGHT		50
#define WDU_SHADOW		51
#define WDU_PAPER_LENGTH_MSB	52
#define WDU_PAPER_LENGTH_LSB	53

/*
 * Since iodone() & devstrat() want a "struct buf *" and we need more
 * than that we define a structure with trailing addition data and
 * use a pointer to that instead of just a buf struct.	IBM does it too
 * with their sc_buf struct which also starts with a buf struct.
 * We tack on our scan_df struct to the whole thing
 */

struct scan_buf {
  struct sc_buf	  scsi_buf;
  struct scan_df *scan_ptr;
  uint	 retry_count;
};

/* structure for the REQUEST SENSE data from the target */

struct scan_req_sense_data {
  uchar err_code;
  uchar segment_number;
  uchar sense_key;
#define ILI_BIT 0x20		/* illegal length bit in sense key field */
#define EOM_BIT 0x40		/* end of media bit in sense key field */
  uchar info0;
  uchar info1;
  uchar info2;
  uchar info3;
  uchar additional_sense_length;
  uchar cmd_specific0;
  uchar cmd_specific1;
  uchar cmd_specific2;
  uchar cmd_specific3;
  uchar additional_sense_code;
  uchar additional_sense_code_qualifier;
  uchar FRU;			/* field replaceable unit code */
  uchar sense_key_specific0;
  uchar sense_key_specific1;
  uchar sense_key_specific2;
};

struct scan_inq_data {
  uchar peripheral_type;
  uchar device_type_qualifier;
  uchar version;
  uchar response_data_format;
  uchar additional_inq_length;
  uchar reserved01;
  uchar reserved02;
  uchar flags;
  uchar vendor_id[16];
  uchar product_id[16];
  uchar product_version[4];
};

/*
 *  structure for misc scanner info
 *   this gets xmalloc'd from the pinned heap in scanconfig()
 */
struct scan_df {
  struct scan_df *next_defined; /* Ptr to next defined scan_df		  */
  struct scan_df *next_opened;	/* Ptr to next opened scan_df		  */
  struct scan_dds s_dds;	/* device dependent info struct		  */
  dev_t	 devno;			/* key used to tie entry to special file  */
  struct file *fp;		/* for fp_devxxx() routines		  */
  struct scan_buf data_buf;	/* buf block to use for normal data xfers */
  struct scan_buf cmd_buf;	/* buf block to use for a command	  */
  struct scan_buf reqsen_buf;	/* buf block to use for a request sense	  */
  struct scan_buf ioctl_buf;	/* buf block to use for a ioctl()s	  */
  struct sc_error_log_df scan_errlog; /* error log structure		  */
  struct scan_req_sense_data rs; /* space for request sense replies	  */
  struct scan_inq_data inq;	/* space for inquiry replies              */
  int    window_size;		/* # of bytes in the window		  */
  int    bytes_ready;		/* response from GET_BUFFER		  */
  int	 tur_count;		/* number of Test Unit Ready's tried	  */
  uchar	 cmd_state;		/* current command used in iodone()	  */
  enum   hp_cmds {
    HP_CONFIG,			/* send the scan params to the scanner    */
    HP_TRIGGER,			/* start the scan operation               */
    HP_PAPER_SENSE,		/* test ADF for paper                     */
    HP_UNLOAD_ADF,		/* unload document from ADF               */
  } hp_command;
  uchar  hp_response[20];	/* escape code response from ScanJet IIc  */
  uchar	 check_condition;	/* set when a request sense is needed	  */
  uchar  adapter_error;		/* set to the ret val from adapter_error()*/
  uchar	 retry;			/* this command is a retry		  */
  uchar  triggered;		/* has a SCSI_SCAN_TRIGGER been sent yet? */
  uchar	 opened;		/* the device has been opened		  */
  uchar  do_unload;		/* toggle for PAPER_PUSH ioctl            */
  uchar  configured;            /* SCSI_SET_WINDOW has been sent	  */
  uchar  using_adf;		/* flag to indicate UMAX ADF read         */
  enum   adf_states {
    ADF_UNUSED,			/* scan from flatbed, not ADF             */
    ADF_ARMED,			/* scan from ADF, everything's set up     */
    ADF_CLEANUP,		/* eject paper from ADF on close          */
  } adf_state;
  uchar  buf_status[12];	/* for the SCSI_GET_BUFFER_STATUS command */
  uchar	 diag_mode;		/* device is open in diag mode		  */
  uchar	 retain_reservation;	/* no release on close			  */
#ifdef IOSTAT
  struct dkstat scan_dkstat;	/* iostat struct for scanner */
#endif
};

/*
 * scanner hash table definition
 */
struct scan_hash_table_df {
	struct scan_df	     *defined_head;
	struct scan_df	     *opened_head;
};


/* function prototypes */

/* top half */
int scanconfig(dev_t devno, int cmd, struct uio *uiop);
int scanopen(dev_t devno, ulong devflag, chan_t chan, int ext);
int scanclose(dev_t devno, chan_t chan);
int scanread(dev_t devno, struct uio *uiop, chan_t chan, int ext);
int scanioctl(dev_t devno, int cmd, int arg,
	      ulong devflag, chan_t chan, int ext);
int calc_umax_row_len(int dpi, int ww);
int umax_has_paper_in_adf(struct scan_df *scan_ptr);
void find_umax(struct scan_df *scan_ptr);
void do_request_sense(struct scan_df *scan_ptr);

/* bottom half */
void scaniodone(struct buf *bp);
void scan_scsi_cmd(uchar, struct scan_df *);
void scanner_reserve_unit(struct scan_df *);
void scanner_release_unit(struct scan_df *);
/* void scanner_request_sense(struct scan_df *); */
void scanner_set_window(struct scan_df *);
void scan_trigger(struct scan_df *);
void scanwait(struct buf *);	/* XXX -> should use iowait() if possible */


uchar adapter_error(struct scan_buf *);

extern int nodev();		/* returns error */
extern int nulldev();		/* returns success */
extern int uphysio(struct uio *, int, uint, dev_t, int (*)(), int(*)(), void *);
extern int uiomove(caddr_t, int, enum uio_rw, struct uio *);
extern int devstrat(struct buf *);
