/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* 
 * unconfigure method for PINT
 * third release, version 0.3
 */

#include <sys/types.h> 
#include <sys/cfgdb.h> 
#include <sys/cfgodm.h>
#include <sys/sysconfig.h>
#include <sys/device.h>
#include <sys/mode.h>
#include <sys/errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <cf.h>

#define USAGE(S)      (fprintf(stderr, "Usage: %s -l name\n", (S)), exit(E_ARGS))

int		lock_id;

close_odm_exit(err)
{
  odm_unlock(lock_id);
  odm_terminate();

  exit(err);
}


main(int argc, char *argv[])
{
  struct cfg_dd	cfg;
  struct cfg_load unload;
  struct PdDv	pddv;
  struct CuDv	cudv;
  char	 search_string[0x100];
  char   *logical_name = NULL;
  char   *odm_path;		/* the ODM directory */
  char   odm_lock_path[1024];	/* the ODM configure lock file */
  char   devpath[0x100];
  long	 majorno, minor;
  int    how_many;
  int    c;			/* for command line processing */

  extern int   optind;		/* for getopt() */
  extern char *optarg;		/* ditto */


  /****  command line argument processing section  ***/

  while ((c = getopt(argc, argv, "l:")) != EOF) {
    switch (c) {
    case 'l':
      if (logical_name != NULL)
	USAGE(*argv);
      logical_name = optarg;
      break;
    default:
      USAGE(*argv);
    }
  }

  if (logical_name == NULL) /* must have a logical device name */
    USAGE(*argv);


  /****   bring up & lock the ODM  ***/

  if (odm_initialize() < 0)
    exit(E_ODMINIT);

  if ((odm_path = getenv("ODMDIR")) == NULL) {
    fprintf(stderr, "%s: ODMDIR environment variable is undefined.\n", argv[0]);
    exit(E_ODMLOCK);
  }
  strcpy(odm_lock_path, odm_path);
  strcat(odm_lock_path, "/config_lock");
  if ((lock_id = odm_lock(odm_lock_path, 10)) == -1)
    close_odm_exit(E_ODMLOCK);


  /****  get the CuDv entry with our logical_name  ***/

  sprintf(search_string, "name = '%s'", logical_name);
  switch ((int)odm_get_first(CuDv_CLASS, search_string, &cudv))	{
  case 0:
    close_odm_exit(E_NOCuDv);
  case -1:
    close_odm_exit(E_ODMGET);
  }


  /**** if the device is in state DEFINED, then no unconfigure work to do   ***/

  if (cudv.status == (short)DEFINED)
    close_odm_exit(E_OK);


  /****   get the PreDefined data from the ODM  ***/

  sprintf(search_string, "uniquetype = '%s'", cudv.PdDvLn_Lvalue);
  switch ((int)odm_get_first(PdDv_CLASS, search_string, &pddv))	{
  case 0:
    close_odm_exit(E_NOPdDv);
  case -1:
    close_odm_exit(E_ODMGET);
  }


  /****   remove the special file   ***/

  sprintf(devpath, "/dev/%s", cudv.name);
  if (unlink(devpath) < 0 && errno != ENOENT)
    close_odm_exit(E_RMSPECIAL);

  /*   get the kmid   */

  unload.path = "/etc/drivers/scandd";
  unload.libpath = "";
  unload.kmid = 0;

  if (sysconfig(SYS_QUERYLOAD, &unload, sizeof(unload)) == CONF_FAIL) {
    fprintf(stderr, "sysconfig: query failed %d\n", errno);
    exit(errno);
  }

  if (*pddv.DvDr != 0) {

    if ((majorno = genmajor(pddv.DvDr)) == -1)
      close_odm_exit(E_MAJORNO);

    /* your minor is your logical name's number */

    minor = atol(&logical_name[4]);
    
    cfg.devno  = makedev(majorno, minor);
    cfg.kmid   = unload.kmid;
    cfg.ddsptr = NULL;
    cfg.ddslen = 0;
    cfg.cmd    = CFG_TERM;

    if (sysconfig(SYS_CFGDD, &cfg, sizeof(struct cfg_dd)) == -1)
      close_odm_exit(E_CFGTERM);

    if (loadext(pddv.DvDr, FALSE, FALSE) == NULL)
      close_odm_exit(E_UNLOADEXT);
  } else {
#ifdef DEBUG
    fprintf(stderr, "ucfgscan: *pddv.DvDr == 0\n");
#endif
  }

  cudv.status = DEFINED;
  if (odm_change_obj(CuDv_CLASS, &cudv) < 0)
    close_odm_exit(E_ODMUPDATE);

  close_odm_exit(E_OK);
}
