/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Attempt to modify "conf" file automatically.
 * Maybe this should be done using a scripting language like awk, sed or perl.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

char *prog_name;

char *readconf(filename)
char *filename;
{
  struct stat stat_buf;
  char *buf;
  int fd;

  if (stat(filename, &stat_buf) < 0) {
    perror("stat failed");
    fprintf(stderr, "%s: can't stat %s", prog_name, filename);
    exit(0);
  }

  if ((buf = malloc(stat_buf.st_size + 1)) == NULL) {
    perror("malloc failed");
    fprintf(stderr, "%s: malloc failed", prog_name);
    exit(0);
  }

  if ((fd = open(filename, O_RDONLY)) < 0) {
    perror("open failed");
    fprintf(stderr, "%s: can't open %s", prog_name, filename);
    exit(0);
  }

  if (read(fd, buf, stat_buf.st_size) != stat_buf.st_size) {
    perror("read failed");
    fprintf(stderr, "%s: can't read %s", prog_name, filename);
    exit(0);
  }

  buf[stat_buf.st_size] = '\0';
  close(fd);
  return(buf);
}

void write_defines(out)
FILE *out;
{
  fprintf(out, "#include \"scan.h\"\n");
  fprintf(out, "#if NSCAN > 0\n");
  fprintf(out, "extern int scanopen(), scanclose(), scanread(), scanioctl();\n");
  fprintf(out, "#else\n");
  fprintf(out, "#define scanopen  nodev\n");
  fprintf(out, "#define scanclose nodev\n");
  fprintf(out, "#define scanread  nodev\n");
  fprintf(out, "#define scanioctl nodev\n");
  fprintf(out, "#endif\n\n");
}

void write_cdevsw_stuff(out, n)
FILE *out;
int n;
{
  fprintf(out, "    {\n");
  fprintf(out, "\tscanopen,\tscanclose,\tscanread,\tnodev,\t\t/*%d*/\n", n);
  fprintf(out, "\tscanioctl,\tnodev,\t\tnodev,\t\t0,\n");
  fprintf(out, "\t0,\t\t0,\n");
  fprintf(out, "    },\n");
}

main(argc, argv)
int argc;
char *argv[];
{
  enum actions { append, search } action;
  char numeral[10];
  char *conf_buf;
  char *seltrue_spot;
  char *p, *q;
  FILE *out;
  int last_major;
  int commas;

  if (argc != 4) {
    fprintf(stderr, "%s: bad arguments\n", argv[0]);
    exit(0);
  }

  prog_name = argv[0];

  switch (argv[3][0]) {
  case 'a':
    action = append;
    break;
  case 't':
    action = search;
    break;
  default:
    fprintf(stderr, "%s: bad action (not a or t)\n", argv[0]);
    exit(0);
  }

  if (action == append) {
    if ((out = fopen(argv[2], "w")) == NULL) {
      perror("open failed");
      fprintf(stderr, "%s: can't open %s for writing\n", argv[0], argv[2]);
      exit(0);
    }
  }

  conf_buf = readconf(argv[1]);

  if ((seltrue_spot = strstr(conf_buf, "extern int seltrue")) == NULL) {
    fprintf(stderr, "%s: can't find 'extern int seltrue' in %s\n", argv[0], argv[1]);
    exit(0);
  }

  if (action == append) {
    *seltrue_spot = '\0';
    fprintf(out, "%s", conf_buf);
    *seltrue_spot = 'e';
    write_defines(out);
  }

  p = strstr(seltrue_spot, "struct cdevsw");
  p = strstr(p, "};");
  commas = 8;
  while (commas) {
    --p;
    if (*p == ',')
      --commas;
  }
  p = strstr(p, "/*");
  p += 2;
  q = numeral;
  while (isdigit(*p))
    *q++ = *p++;
  sscanf(numeral, "%d", &last_major);

  if (action == append) {
    p = strstr(p, "}");
    ++p;
    *p = '\0';
    fprintf(out, "%s,\n", seltrue_spot);
    ++p;
    ++last_major;
    write_cdevsw_stuff(out, last_major);
    fprintf(out, "%s", p);
    fclose(out);
  }

  exit(last_major);
}
