/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * scandd.h include file for PINT scanner driver for SunOS 4.1
 * third release version 0.3
 */

/************************************************************************
 *									*
 * Defines for SCSI generic device driver				*
 *									*
 ***********************************************************************/

/*
 * Define various buffer and I/O block sizes
 */

#define SECSIZE			512		/* Bytes/sector */
#define SECDIV			9		/* log2 (SECSIZE) */
#define UN_TMPBUF_SIZE		SECSIZE		/* For internal DVMA */

/************************************************************************
 *									*
 * Private data structure. Pointed to by the un_private pointer		*
 * of one of the SCSI_DEVICE structures.				*
 *									*
 ************************************************************************/

struct scsi_scan {
  struct  scsi_device *un_sd;	/* back pointer to SCSI_DEVICE */
  struct  scsi_pkt *un_rsp;	/* ptr to request sense command pkt */
  struct  buf *un_rbufp;	/* for use in raw io */
  struct  buf *un_sbufp;	/* for use in special io */
  u_int	  un_err_resid;		/* resid from last error */
  u_char  un_err_severe;	/* error severity */
  u_char  un_status;		/* sense key from last error */
  u_char  un_err_code;		/* vendor unique error code  */
  u_char  un_retries;		/* retry count */
  u_char  un_state;		/* current state */
  u_char  un_last_state;	/* last state */
  u_int	  un_attached;		/* device attached flag */
  u_char  un_triggered;		/* set window, & read trigger sent */
  u_short un_bufcnt;		/* count of bufs on un_quehd */
  u_long  window_size;		/* size of scanner active area in bytes */
  struct  buf *un_quehd;	/* list of pending requests */
  struct  buf *un_quetl;	/* last buf on queue */
  struct  buf *un_active;	/* currently active request */
  struct  scancmd un_ucmd;	/* for SCANIOC_CMD */
  struct  scan_io sio;		/* scanner parameters */
  u_char  using_adf;		/* umax is/was using ADF for read */
  u_char  buf_status[3];	/* for Ricoh SCMD_GET_BUFFER_STATUS command */
  enum   adf_states {
    ADF_UNUSED,			/* scan from flatbed, not ADF             */
    ADF_ARMED,			/* scan from ADF, everything's set up     */
    ADF_CLEANUP,		/* eject paper from ADF on close          */
  } adf_state;
  caddr_t un_tmpbuf;		/* local buffer */
};

/* defines for image modes */

#define BINARY_MONOCHROME   0
#define DITHERED_MONOCHROME 1
#define GRAYSCALE 2
#define COLOR 5
#define RED   103
#define GREEN 104
#define BLUE  105

#define SCANNING_MEASUREMENT_PARAMETERS (uchar)(3)
#define ADF_CONTROL (uchar)(0x26)

/* sizes for mode parameter's base_measurement_unit */

#define INCHES 0
#define MILLIMETERS 1
#define POINTS 2

/*
 *  SCSI_MODE_SELECT parameter list
 */
struct ricoh_mode_params {
  /* header */
  uchar mode_data_length;
  uchar medium_type;
  uchar device_specific_parameter;
  uchar block_descriptor_length;

  /* page descriptor */
  uchar page_code;
  uchar paramter_length;
  uchar base_measurement_unit;
  uchar reserved1;
  uchar measurement_unit_devisor_msb;
  uchar measurement_unit_devisor_lsb;
  uchar reserved2;
  uchar reserved3;

  /* ADF support */
  uchar adf_page_code;
  uchar adf_parameter_length;
  uchar adf_control;
  uchar reserved4;
  uchar reserved5;
  uchar reserved6;
  uchar reserved7;
  uchar reserved8;
};

/*
 * Driver states
 */

#define	SCAN_STATE_NIL		0
#define	SCAN_STATE_CLOSED	1
#define	SCAN_STATE_OPENING	2
#define	SCAN_STATE_OPEN		3
#define	SCAN_STATE_SENSING	4
#define	SCAN_STATE_RWAIT	5
#define	SCAN_STATE_DETACHED	6

/*
 * Error levels
 */

#define	SCANERR_ALL		0
#define	SCANERR_UNKNOWN		1
#define	SCANERR_INFORMATIONAL	2
#define	SCANERR_RECOVERED	3
#define	SCANERR_RETRYABLE	4
#define	SCANERR_FATAL		5

/*
 * scanintr codes
 */

#define	COMMAND_DONE		0
#define	COMMAND_DONE_ERROR	1
#define	QUE_COMMAND		2
#define	QUE_SENSE		3
#define	JUST_RETURN		4

/*
 * scanner specific commands
 */
#define SCMD_TRIGGER_SCAN       0x1b
#define SCMD_SET_WINDOW         0x24
#define SCMD_OBJECT_POSITION    0x31
#define SCMD_GET_BUFFER_STATUS  0x34
/*
 * Special fake command to tag a user defined command
 */
#define SCMD_UCMD	0xFF

/*
 *  ADF codes
 */
#define UNLOAD_ADF 0
#define LOAD_ADF 1
