/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * fs1toppm.c: convert output from Ricoh FS1 into PPM format
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

char fp_buf[262144];
char out_buf[262144];

void usage();
int read_rgb(FILE *, char *, int);

main(int argc, char *argv[])
{
  int width;
  FILE *fp;
  char rgb[3];

  if (argc != 4) {
    usage();
    exit(1);
  }

  width = atoi(argv[1]);

  if ((fp = fopen(argv[3], "r")) == NULL) {
    perror("open");
    exit(-1);
  }
  setbuffer(fp, fp_buf, 262144);
  setbuffer(stdout, out_buf, 262144);

  printf("P6\n%d %d 255\n", width, atoi(argv[2]));

  while(read_rgb(fp, rgb, width) != EOF) {
    fwrite(rgb, 3, 1, stdout);
  }

  fclose(fp);
}

int read_rgb(FILE *fp, char *rgb, int width)
{
  static int cur_pos = 0;

  if (fseek(fp, cur_pos, SEEK_SET) != 0) {
    perror("fseek red");
    exit(-1);
  }
  if (fread(&rgb[0], 1, 1, fp) != 1)
    return EOF;

  if (fseek(fp, cur_pos + width, SEEK_SET) != 0) {
    perror("fseek green");
    exit(-1);
  }
  if (fread(&rgb[1], 1, 1, fp) != 1)
    return EOF;

  if (fseek(fp, cur_pos + width * 2, SEEK_SET) != 0) {
    perror("fseek blue");
    exit(-1);
  }
  if (fread(&rgb[2], 1, 1, fp) != 1)
    return EOF;

  if (++cur_pos % width == 0)
    cur_pos += width * 2;

  return 0;
}

void usage()
{
  fprintf(stderr, "usage: fs1toppm width height fs1_file\n");
}
