/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * get_pixel_size.c: return the width and height of the specified scanner's
 *		     active area
 */

#include <stdio.h>
#include <fcntl.h>
#include <sys/scanio.h>

void err_exit(char *progname)
{
  fprintf(stderr, "usage: %s [-l scan0]\n", progname);
  exit(1);
}

main(int argc, char *argv[])
{
  int c;
  extern int optind;
  extern char *optarg;
  char scan_name[80];
  char *scan_logical_name = "scan0";
  int scan_fd;
  char err_string[128];
  struct scan_pixel_sizes pix;

  while ((c = getopt(argc, argv, "l:")) != EOF) {
    switch (c) {
    case 'l':
      scan_logical_name = optarg;
      break;
    case '?':
      err_exit(argv[0]);
    }
  }

  strcpy(scan_name, "/dev/");
  strcat(scan_name, scan_logical_name);

  if ((scan_fd = open(scan_name, O_RDONLY)) < 0) {
    sprintf(err_string, "%s: open of scanner (%s) failed",
	    argv[0], scan_name);
    perror(err_string);
    err_exit(argv[0]);
  }

  if (ioctl(scan_fd, SCAN_GET_PIXEL_SIZE, &pix) < 0) {
    sprintf(err_string, "%s: ioctl SCAN_GET_PIXEL_SIZE failed", argv[0]);
    perror(err_string);
    err_exit(argv[0]);
  }

  close(scan_fd);

  printf("%d %d\n", pix.pix_width, pix.pix_height);

  exit(0);
}
