/*
 *  PINT Pint Is Not TWAIN - common scanner driver interface for UN*X
 *  Copyright (C) 1994 Kenneth Stailey ken@spacenet.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * get_scanner.c: retrieve the a scanner's parameters
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/scanio.h>

void
usage(char *prog_name)
{
  fprintf(stderr, "usage: %s [-q][-l <logical name>]\n", prog_name);
  exit(-1);
}


main(int argc, char *argv[])
{
  int s_fd;
  struct scan_io s_io;

  int quiet = FALSE;

  char *logical_name = "scan0";
  char device_special[255];

  int c;
  extern int optind;
  extern char *optarg;

  while ((c = getopt(argc, argv, "ql:")) != EOF) {
    switch (c) {
    case 'q':
      quiet = TRUE;
      break;

    case 'l':
      logical_name = optarg;
      break;

    case '?':
      usage(argv[0]);
      break;
    }
  }

  sprintf(device_special, "/dev/%s", logical_name);
  if ((s_fd = open(device_special, O_RDONLY)) < 0) {
    perror("open of scanner");
    exit(-1);
  }

  if (ioctl(s_fd, SCAN_GET, &s_io) < 0) {
    perror("ioctl on scanner");
    exit(-1);
  }

  if (quiet) {
    printf("-w %g -h %g -x %g -y %g -b %d -c %d -r %d",
	   (double)s_io.scan_window_width / 1200.0,
	   (double)s_io.scan_window_length / 1200.0,
	   (double)s_io.scan_x_origin / 1200.0,
	   (double)s_io.scan_y_origin / 1200.0,
	   s_io.scan_brightness,
	   s_io.scan_contrast,
	   s_io.scan_x_resolution);
    switch (s_io.scan_image_mode) {
    case SIM_BINARY_MONOCHROME:
      printf(" -i m\n");
      break;
    case SIM_DITHERED_MONOCHROME:
      printf(" -i d\n");
      break;
    case SIM_GRAYSCALE:
      printf(" -i g\n");
      break;
    case SIM_COLOR:
      printf(" -i c\n");
      break;
    case SIM_RED:
      printf(" -i R\n");
      break;
    case SIM_GREEN:
      printf(" -i G\n");
      break;
    case SIM_BLUE:
      printf(" -i B\n");
      break;
    }
  } else {
    printf("width = %g inches\n", (double)s_io.scan_window_width / 1200.0);
    printf("height = %g inches\n", (double)s_io.scan_window_length / 1200.0);
    printf("x_origin = %g inches\n", (double)s_io.scan_x_origin / 1200.0);
    printf("y_origin = %g inches\n", (double)s_io.scan_y_origin / 1200.0);
    printf("brightness = %d\n", s_io.scan_brightness);
    printf("contrast = %d\n", s_io.scan_contrast);
    printf("resolution = %d dpi\n", s_io.scan_x_resolution);
    printf("image_mode = ");
    switch (s_io.scan_image_mode) {
    case SIM_BINARY_MONOCHROME:
      printf("binary_monochrome\n");
      break;
    case SIM_DITHERED_MONOCHROME:
      printf("dithered_monochrome\n");
      break;
    case SIM_GRAYSCALE:
      printf("grayscale\n");
      break;
    case SIM_COLOR:
      printf("color\n");
      break;
    case SIM_RED:
      printf("red\n");
      break;
    case SIM_GREEN:
      printf("green\n");
      break;      
    case SIM_BLUE:
      printf("blue\n");
      break;      
    }
  }

  close(s_fd);
}
